#include <MFRC522.h>

MFRC522 rfid(10, 9);
MFRC522::MIFARE_Key key = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
MFRC522::StatusCode status;
char buffer[49];
int bufferIndex = 0;
bool isBufferReady = false;

void setup() {
  Serial.begin(9600);
  SPI.begin();
  rfid.PCD_Init();
  Serial.println(F("Inserisci il testo (max 48 caratteri):"));
}

void loop() {
  while (Serial.available() && !isBufferReady) {
    char c = Serial.read();
    if (bufferIndex < 48 && c != '\n') {
      buffer[bufferIndex++] = c;
    } else {
      buffer[bufferIndex] = '\0';
      isBufferReady = true;
      Serial.println(buffer);
      Serial.println();
      Serial.println(F("Avvicina la tessera RFID per la scrittura"));
    }
  }

  if (isBufferReady && rfid.PICC_IsNewCardPresent() && rfid.PICC_ReadCardSerial()) {
    Serial.println(F("Scrittura in corso..."));
    if (writeCard(buffer, bufferIndex)) {
      Serial.println(F("Scrittura completata con successo!"));
    }

    rfid.PICC_HaltA();
    rfid.PCD_StopCrypto1();

    isBufferReady = false;
    bufferIndex = 0;
    while (Serial.available()) {
      Serial.read();
    }
    Serial.println();
    Serial.println(F("Inserisci il testo (max 48 caratteri):"));
  }
}

bool writeCard(char *content, int contentLength) {
  int index = 0;
  byte blockData[16];

  for (byte blockOffset = 0; blockOffset < 3; blockOffset++) {
    memset(blockData, '\0', 16);
    index = min(16, contentLength - blockOffset * 16);
    if (index > 0) {
      strncpy((char *)blockData, content + blockOffset * 16, index);
    }
    if (!writeBlock(4 + blockOffset, blockData)) {
      return false;
    }
  }

  return true;
}

bool writeBlock(byte blockAddr, byte *blockData) {
  status = rfid.PCD_Authenticate(
    MFRC522::PICC_CMD_MF_AUTH_KEY_A,
    blockAddr,
    &key,
    &(rfid.uid));
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("Autenticazione del blocco non riuscita: "));
    Serial.println(blockAddr);
    Serial.println(rfid.GetStatusCodeName(status));
    return false;
  }

  status = rfid.MIFARE_Write(blockAddr, blockData, 16);
  if (status != MFRC522::STATUS_OK) {
    Serial.print(F("Errore durante la scrittura del blocco: "));
    Serial.println(blockAddr);
    Serial.println(rfid.GetStatusCodeName(status));
    return false;
  }

  return true;
}

